--[========================================================================[--

Get Ready screen for Thrust II Reloaded.

Copyright © 2015 Pedro Gimeno Fortea

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

--]========================================================================]--

local getready = {}

local timer
local wait_time = 1.2
local scroll_in_time = 1.2
local quad, prev_x, canvas

function getready.load()
  quad = lg.newQuad(0, 0, 1, 1, main.ww, main.wh) -- to be modified
  canvas = lg.newCanvas(main.ww, main.wh) -- for scrolling
end

function getready.activate()
  if getready.fromstart then
    timer = -wait_time
  else
    timer = 0
  end
  prev_x = main.ww + 8
end

function getready.update(dt)
  timer = timer + dt
  if timer >= scroll_in_time then
    main.activate(game)
    return
  end
end

function getready.draw()
  local x = (main.ww + 8)

  if timer > 0 then
    x = math.floor(x * (1 - timer/scroll_in_time))
  end

  if getready.fromstart and timer < 0 then
    lg.setColor(0,0,0,255)
    lg.rectangle("fill", 0, 0, main.ww, main.wh)
    lg.setColor(255,255,255,255)
    main.centertext("GET READY", main.ww/2, main.wh/2)
    lti.sleep(0.02)
  end

  if timer >= 0 then
    -- Drawing part of a canvas onto itself causes artifacts,
    -- therefore we draw to our local canvas and copy the result back.
    lg.setCanvas(canvas)
    if canvas.clear then
      canvas:clear()
    else
      lg.clear()
    end

    if x >= 0 then
      quad:setViewport((prev_x-x), 0, main.ww-(prev_x-x), main.wh)
      lg.draw(main.canvas, quad, 0, 0)
      prev_x = x
    end
    if x <= main.ww then
      lg.setScissor(x, 0, main.ww-x, main.wh)

      local vpx = math.floor(game.state.ship.x+0.5-main.ww/2)
      local vpy = math.floor(game.state.ship.y+0.5-main.wh/2)
      if vpy < 0 then vpy = 0 end
      if vpy > 2048-main.wh then vpy = 2048-main.wh end
      game.tiles_draw(vpx-x, vpy)
      game.orbs_draw(vpx-x, vpy)

      lg.setScissor()
    end
    lg.setColor(120,150,150,255)
    lg.rectangle("fill", x-8, 0, 8, main.wh)
    lg.setColor(255,255,255,255)

    -- Draw back to the main canvas
    lg.setCanvas(main.canvas)
    lg.draw(canvas)
  end
end

return getready
